"use client"

import { Icons } from "@/components/icons"
import { AspectRatio } from "@/components/ui/aspect-ratio"
import { Button } from "@/components/ui/button"
import { Card } from "@/components/ui/card"
import {
    Dialog,
    DialogContent,
    DialogDescription,
    DialogFooter,
    DialogHeader,
    DialogTitle,
    DialogTrigger,
} from "@/components/ui/dialog"
import { Form, FormControl, FormField, FormItem, FormMessage } from "@/components/ui/form"
import { Input } from "@/components/ui/input"
import { Label } from "@/components/ui/label"
import { Textarea } from "@/components/ui/textarea"
import { toast } from "@/components/ui/use-toast"
import { UserAvatar } from "@/components/user-avatar"
import { zodResolver } from "@hookform/resolvers/zod"
import Image from "next/image"
import { useRouter } from "next/navigation"
import { useEffect, useRef, useState } from "react"
import { useForm } from "react-hook-form"
import { z } from "zod"
import { useUpdateProfile } from "../hooks/use-update-profile"
import { IProfile, IUser } from "../types"

const FormSchema = z.object({
    name: z.string().min(1, "Name can't be blank").max(50, "Name can't be more than 50 characters"),
    bio: z.string().max(160, "Bio can't be more than 160 characters").optional(),
    location: z.string().max(30, "Location can't be more than 30 characters").optional(),
    website: z.string().max(100, "Website can't be more than 100 characters").optional(),
})

const ImagesSchema = z.custom<File>().refine((file) => file instanceof File, { message: "Expected a file" })
    .refine((file) => file?.size < 4000000, { message: "Images must be less than 4MB" })
    .optional()

export const EditProfileModal = ({ user }: { user: IUser }) => {
    const { isLoading, isSuccess, mutate } = useUpdateProfile()

    const form = useForm<z.infer<typeof FormSchema>>({
        resolver: zodResolver(FormSchema),
        defaultValues: {
            name: user.name,
            bio: user.bio || "",
            location: user.location || "",
            website: user.website || "",
        },
    })

    const [open, setOpen] = useState(false)
    const [imageErrorMessage, setImageErrorMessage] = useState("")
    const [chosenBanner, setChosenBanner] = useState<Partial<IProfile>>({
        banner: { url: user?.banner || "", file: undefined },
    })
    const [chosenImages, setChosenImages] = useState<Partial<IProfile>>({
        image: { url: user?.image || "", file: undefined },
    })

    const bannerInputRef = useRef<HTMLInputElement>(null)
    const imageInputRef = useRef<HTMLInputElement>(null)

    const router = useRouter()

    async function onSave(formData: z.infer<typeof FormSchema>) {
        if (!user) return null

        const profile: IProfile = {
            name: formData.name,
            bio: formData.bio,
            location: formData.location,
            website: formData.website,
            banner: { url: user?.banner || "", file: chosenBanner.banner?.file },
            image: { url: user?.image || "", file: chosenImages.image?.file },
        }
        // TODO dont send everything, only send what has changed
        mutate({
            profile,
            userId: user.id,
        })
    }

    useEffect(() => {
        if (isSuccess) {
            toast({
                description: "Successfully updated profile.",
            })
            setOpen(false)
            router.refresh()
        }
    }, [isSuccess, router])

    const chooseImage = async (event: any, type: string) => {
        const file = event.target.files[0]
        if (!file) return
        try {
            ImagesSchema.parse(file)
            setImageErrorMessage("")
        } catch (error: any) {
            const err = error as z.ZodError
            setImageErrorMessage(err.issues[0].message)
            return
        }

        const reader = new FileReader()

        if (type === "banner" && bannerInputRef.current) {
            bannerInputRef.current.value = ""
            reader.onloadend = () => {
                setChosenBanner({
                    ["banner"]: { url: reader.result as string, file },
                })
            }
        }

        if (type === "image" && imageInputRef.current) {
            imageInputRef.current.value = ""
            reader.onloadend = () => {
                setChosenImages({
                    ["image"]: { url: reader.result as string, file },
                })
            }
        }

        reader.readAsDataURL(file)
    }

    if (!user) return null

    return (
        <Dialog open={open} onOpenChange={setOpen}>
            <DialogTrigger asChild>
                <Button variant="outline">Edit Profile</Button>
            </DialogTrigger>
            <DialogContent className="sm:max-w-[600px]">
                <DialogHeader>
                    <DialogTitle>Edit profile</DialogTitle>
                    <DialogDescription>
                        Make changes to your profile here. Click save when you are done.
                    </DialogDescription>
                </DialogHeader>
                <Form {...form}>
                    <form onSubmit={form.handleSubmit(onSave)}>
                        <div className="grid gap-3 py-3">

                            <div className="space-y-2">
                                <div className="grid grid-cols-4 items-center gap-4">
                                    <div className="col-span-1 relative w-full overflow-hidden items-center">
                                        <div className="object-center object-cover w-full h-full">
                                            {!chosenImages.image?.url ?
                                                <UserAvatar
                                                    user={{ username: user.username, image: null }}
                                                    className="object-center object-cover w-full h-full aspect-square"
                                                />
                                                :
                                                <AspectRatio ratio={1 / 1} className="overflow-hidden rounded-full">
                                                    <Image
                                                        src={chosenImages.image?.url}
                                                        alt={"user image"}
                                                        fill
                                                        priority
                                                        className="object-center object-cover w-full h-full "
                                                    />
                                                </AspectRatio>
                                            }
                                        </div>
                                        <input
                                            className="hidden resize-none"
                                            type="file"
                                            accept="image/*"
                                            ref={imageInputRef}
                                            onChange={(e) => chooseImage(e, "image")}
                                            disabled={isLoading}
                                        />
                                        <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 flex space-x-3">
                                            <Button
                                                type="button"
                                                variant="outline"
                                                size="icon"
                                                className="bg-opacity-50 dark:bg-opacity-50"
                                                onClick={() => imageInputRef?.current?.click()}
                                                disabled={isLoading}
                                            >
                                                <Icons.camera />
                                            </Button>
                                        </div>
                                    </div>

                                    <Card className="col-span-3 relative w-full overflow-hidden border-none">

                                        <AspectRatio ratio={3 / 1} className="bg-muted overflow-hidden">
                                            {chosenBanner.banner?.url &&
                                                <Image
                                                    src={chosenBanner.banner?.url}
                                                    alt={"user banner image"}
                                                    fill
                                                    priority
                                                    className="object-center object-cover w-full h-full"
                                                />
                                            }
                                        </AspectRatio>
                                        <input
                                            className="hidden w-full resize-none"
                                            type="file"
                                            accept="image/*"
                                            ref={bannerInputRef}
                                            onChange={(e) => chooseImage(e, "banner")}
                                            disabled={isLoading}
                                        />
                                        <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 flex space-x-3">
                                            <Button
                                                type="button"
                                                variant="outline"
                                                size="icon"
                                                className="bg-opacity-50 dark:bg-opacity-50"
                                                onClick={() => bannerInputRef.current?.click()}
                                                disabled={isLoading}
                                            >
                                                <Icons.camera />
                                            </Button>
                                            {chosenBanner.banner?.url && (
                                                <Button
                                                    type="button"
                                                    variant="outline"
                                                    size="icon"
                                                    className="bg-opacity-50 dark:bg-opacity-50"
                                                    onClick={() => setChosenBanner({ banner: { url: "", file: undefined } })}
                                                    disabled={isLoading}
                                                >
                                                    <Icons.close />
                                                </Button>
                                            )}
                                        </div>
                                    </Card>
                                </div>
                                {imageErrorMessage &&
                                    <div className="grid grid-cols-4 items-center gap-4">
                                        <div className="col-span-1"></div>
                                        <div className="col-span-3">
                                            <p className="text-sm font-medium text-destructive">{imageErrorMessage}</p>
                                        </div>
                                    </div>
                                }
                            </div>

                            <div className="grid grid-cols-4 items-center gap-4">
                                <Label htmlFor="name" className="text-right">
                                    Name
                                </Label>
                                <FormField
                                    control={form.control}
                                    name="name"
                                    render={({ field }) => (
                                        <FormItem className="col-span-3">
                                            <FormControl>
                                                <Input id="name" disabled={isLoading} {...field} />
                                            </FormControl>
                                            <FormMessage />
                                        </FormItem>
                                    )}
                                />
                            </div>

                            <div className="grid grid-cols-4 items-center gap-4">
                                <Label htmlFor="bio" className="text-right">
                                    Bio
                                </Label>
                                <FormField
                                    control={form.control}
                                    name="bio"
                                    render={({ field }) => (
                                        <FormItem className="col-span-3">
                                            <FormControl>
                                                <Textarea id="bio" className="resize-none" disabled={isLoading} {...field} />
                                            </FormControl>
                                            <FormMessage />
                                        </FormItem>
                                    )}
                                />
                            </div>

                            <div className="grid grid-cols-4 items-center gap-4">
                                <Label htmlFor="location" className="text-right">
                                    Location
                                </Label>
                                <FormField
                                    control={form.control}
                                    name="location"
                                    render={({ field }) => (
                                        <FormItem className="col-span-3">
                                            <FormControl>
                                                <Input id="location" disabled={isLoading} {...field} />
                                            </FormControl>
                                            <FormMessage />
                                        </FormItem>
                                    )}
                                />
                            </div>

                            <div className="grid grid-cols-4 items-center gap-4">
                                <Label htmlFor="website" className="text-right">
                                    Website
                                </Label>
                                <FormField
                                    control={form.control}
                                    name="website"
                                    render={({ field }) => (
                                        <FormItem className="col-span-3">
                                            <FormControl>
                                                <Input id="website" disabled={isLoading} {...field} />
                                            </FormControl>
                                            <FormMessage />
                                        </FormItem>
                                    )}
                                />
                            </div>
                        </div>
                        <DialogFooter>
                            <Button type="submit" disabled={isLoading || !form.formState.isDirty}>
                                {isLoading && (
                                    <Icons.spinner className="mr-2 h-4 w-4 animate-spin" />
                                )}
                                Save changes
                            </Button>
                        </DialogFooter>
                    </form>
                </Form>
            </DialogContent>
        </Dialog>
    )
}